diff -r -u asterisk-1.4.10.1/channels/chan_sip.c asterisk-1.4.10.1-wb/channels/chan_sip.c
--- asterisk-1.4.10.1/channels/chan_sip.c	2007-08-09 13:52:43.000000000 -0600
+++ asterisk-1.4.10.1-wb/channels/chan_sip.c	2007-08-22 20:12:40.000000000 -0600
@@ -6240,7 +6240,7 @@
 		ast_build_string(a_buf, a_size, "a=fmtp:%d 0-16\r\n", rtp_code);
 }
 
-#define SDP_SAMPLE_RATE(x) (x == AST_FORMAT_G722) ? 16000 : 8000
+#define SDP_SAMPLE_RATE(x) (x == AST_FORMAT_G722 || x == AST_FORMAT_SPEEXWB_WENGO) ? 16000 : 8000
 
 /*! \brief Add Session Description Protocol message */
 static enum sip_result add_sdp(struct sip_request *resp, struct sip_pvt *p)
diff -r -u asterisk-1.4.10.1/codecs/codec_speex.c asterisk-1.4.10.1-wb/codecs/codec_speex.c
--- asterisk-1.4.10.1/codecs/codec_speex.c	2007-05-24 09:14:02.000000000 -0600
+++ asterisk-1.4.10.1-wb/codecs/codec_speex.c	2007-08-22 20:15:33.000000000 -0600
@@ -372,6 +372,34 @@
 	.buf_size = BUFFER_SAMPLES * 2, /* XXX maybe a lot less ? */
 };
 
+static struct ast_translator speexwbtolin = {
+	.name = "speexwbtolin", 
+	.srcfmt = AST_FORMAT_SPEEXWB_WENGO,
+	.dstfmt =  AST_FORMAT_SLINEAR,
+	.newpvt = speextolin_new,
+	.framein = speextolin_framein,
+	.destroy = speextolin_destroy,
+	.sample = speextolin_sample,
+	.desc_size = sizeof(struct speex_coder_pvt),
+	.buffer_samples = BUFFER_SAMPLES,
+	.buf_size = BUFFER_SAMPLES * 2,
+	.native_plc = 1,
+};
+
+static struct ast_translator lintospeexwb = {
+	.name = "lintospeexwb", 
+	.srcfmt = AST_FORMAT_SLINEAR,
+	.dstfmt = AST_FORMAT_SPEEXWB_WENGO,
+	.newpvt = lintospeex_new,
+	.framein = lintospeex_framein,
+	.frameout = lintospeex_frameout,
+	.destroy = lintospeex_destroy,
+	.sample = lintospeex_sample,
+	.desc_size = sizeof(struct speex_coder_pvt),
+	.buffer_samples = BUFFER_SAMPLES,
+	.buf_size = BUFFER_SAMPLES * 2, /* XXX maybe a lot less ? */
+};
+
 static void parse_config(void) 
 {
 	struct ast_config *cfg = ast_config_load("codecs.conf");
@@ -495,6 +523,8 @@
 
 	res = ast_unregister_translator(&lintospeex);
 	res |= ast_unregister_translator(&speextolin);
+	res |= ast_unregister_translator(&lintospeexwb);
+	res |= ast_unregister_translator(&speexwbtolin);
 
 	return res;
 }
@@ -505,10 +535,25 @@
 
 	parse_config();
 	res=ast_register_translator(&speextolin);
-	if (!res) 
+	if (!res) {
 		res=ast_register_translator(&lintospeex);
-	else
-		ast_unregister_translator(&speextolin);
+		if (!res) {
+			res=ast_register_translator(&speexwbtolin);
+			if (!res) {
+				res=ast_register_translator(&lintospeexwb);
+				if (res) {
+                        		ast_unregister_translator(&speextolin);
+                        		ast_unregister_translator(&lintospeex);
+					ast_unregister_translator(&speexwbtolin);
+				}
+			} else {
+                        	ast_unregister_translator(&speextolin);
+                        	ast_unregister_translator(&lintospeex);
+			}
+		} else {
+	                ast_unregister_translator(&speextolin);
+		}
+	}	
 
 	return res;
 }
diff -r -u asterisk-1.4.10.1/formats/format_pcm.c asterisk-1.4.10.1-wb/formats/format_pcm.c
--- asterisk-1.4.10.1/formats/format_pcm.c	2007-06-29 14:56:18.000000000 -0600
+++ asterisk-1.4.10.1-wb/formats/format_pcm.c	2007-08-22 20:17:21.000000000 -0600
@@ -455,6 +455,18 @@
 	.buf_size = (BUF_SIZE * 2) + AST_FRIENDLY_OFFSET,
 };
 
+static const struct ast_format speexwb_wengo_f = {
+        .name = "speexwb_wengo",
+        .exts = "speexwb_wengo",
+        .format = AST_FORMAT_SPEEXWB_WENGO,
+        .write = pcm_write,
+        .seek = pcm_seek,
+        .trunc = pcm_trunc,
+        .tell = pcm_tell,
+        .read = pcm_read,
+        .buf_size = (BUF_SIZE * 2) + AST_FRIENDLY_OFFSET,
+};
+
 static const struct ast_format au_f = {
 	.name = "au",
 	.exts = "au",
@@ -482,7 +494,8 @@
 	return ast_format_register(&pcm_f)
 		|| ast_format_register(&alaw_f)
 		|| ast_format_register(&au_f)
-		|| ast_format_register(&g722_f);
+		|| ast_format_register(&g722_f)
+		|| ast_format_register(&speexwb_wengo_f);
 }
 
 static int unload_module(void)
@@ -490,7 +503,8 @@
 	return ast_format_unregister(pcm_f.name)
 		|| ast_format_unregister(alaw_f.name)
 		|| ast_format_unregister(au_f.name)
-		|| ast_format_unregister(g722_f.name);
+		|| ast_format_unregister(g722_f.name)
+		|| ast_format_unregister(speexwb_wengo_f.name);
 }	
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Raw/Sun uLaw/ALaw 8KHz (PCM,PCMA,AU), G.722 16Khz");
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Raw/Sun uLaw/ALaw 8KHz (PCM,PCMA,AU), G.722 16Khz, SpeeX WB");
diff -r -u asterisk-1.4.10.1/include/asterisk/frame.h asterisk-1.4.10.1-wb/include/asterisk/frame.h
--- asterisk-1.4.10.1/include/asterisk/frame.h	2006-10-30 09:27:34.000000000 -0700
+++ asterisk-1.4.10.1-wb/include/asterisk/frame.h	2007-08-22 20:08:53.000000000 -0600
@@ -242,6 +242,10 @@
 #define AST_FORMAT_G726		(1 << 11)
 /*! G.722 */
 #define AST_FORMAT_G722		(1 << 12)
+/*! Openwengo specific SpeeX wideband */
+#define AST_FORMAT_SPEEXWB_WENGO (1 << 13)
+/*! Maximum audio format number */
+#define AST_FORMAT_MAX_AUDIO_NUM 14
 /*! Maximum audio format */
 #define AST_FORMAT_MAX_AUDIO	(1 << 15)
 /*! Maximum audio mask */
@@ -258,6 +262,8 @@
 #define AST_FORMAT_H263_PLUS	(1 << 20)
 /*! H.264 Video */
 #define AST_FORMAT_H264		(1 << 21)
+/*! Maximum video format number */
+#define AST_FORMAT_MAX_VIDEO_NUM 22
 /*! Maximum video format */
 #define AST_FORMAT_MAX_VIDEO	(1 << 24)
 #define AST_FORMAT_VIDEO_MASK   (((1 << 25)-1) & ~(AST_FORMAT_AUDIO_MASK))
@@ -527,6 +533,9 @@
 /*! \brief Returns the number of samples contained in the frame */
 int ast_codec_get_samples(struct ast_frame *f);
 
+/*! \brief Returns the transmission rate for the frame */
+int ast_codec_get_rate(struct ast_frame *f);
+
 /*! \brief Returns the number of bytes for the number of samples of the given format */
 int ast_codec_get_len(int format, int samples);
 
diff -r -u asterisk-1.4.10.1/main/channel.c asterisk-1.4.10.1-wb/main/channel.c
--- asterisk-1.4.10.1/main/channel.c	2007-08-06 15:41:13.000000000 -0600
+++ asterisk-1.4.10.1-wb/main/channel.c	2007-08-22 20:24:33.000000000 -0600
@@ -684,6 +684,8 @@
 		AST_FORMAT_ALAW,
 		/*! G.722 is better then all below, but not as common as the above... so give ulaw and alaw priority */
 		AST_FORMAT_G722,
+		/*! (OpenWengo) Speex wideband */
+		AST_FORMAT_SPEEXWB_WENGO,
 		/*! Okay, well, signed linear is easy to translate into other stuff */
 		AST_FORMAT_SLINEAR,
 		/*! G.726 is standard ADPCM, in RFC3551 packing order */
diff -r -u asterisk-1.4.10.1/main/frame.c asterisk-1.4.10.1-wb/main/frame.c
--- asterisk-1.4.10.1/main/frame.c	2007-06-20 11:52:57.000000000 -0600
+++ asterisk-1.4.10.1-wb/main/frame.c	2007-08-22 20:10:28.000000000 -0600
@@ -117,7 +117,7 @@
 	{ 1, AST_FORMAT_ILBC, "ilbc", "iLBC", 50, 30, 30, 30, 30 },		/*!< 11: codec_ilbc.c */ /* inc=30ms - workaround */
 	{ 1, AST_FORMAT_G726_AAL2, "g726aal2", "G.726 AAL2", 40, 10, 300, 10, 20 },	/*!<  12: codec_g726.c */
 	{ 1, AST_FORMAT_G722, "g722", "G722"},					/*!< 13 */
-	{ 0, 0, "nothing", "undefined" },
+	{ 1, AST_FORMAT_SPEEXWB_WENGO, "speexwb-wengo", "OpenWengo specific SpeeX wideband" }, /*!< 14: codec_speex.c */
 	{ 0, 0, "nothing", "undefined" },
 	{ 0, 0, "nothing", "undefined" },
 	{ 0, 0, "nothing", "undefined" },
@@ -271,7 +271,7 @@
 		memmove(s->data, s->data + len, s->len);
 		if (!ast_tvzero(s->delivery)) {
 			/* If we have delivery time, increment it, otherwise, leave it at 0 */
-			s->delivery = ast_tvadd(s->delivery, ast_samp2tv(s->f.samples, 8000));
+			s->delivery = ast_tvadd(s->delivery, ast_samp2tv(s->f.samples, ast_codec_get_samples(&s->f)));
 		}
 	}
 	/* Return frame */
@@ -631,7 +631,7 @@
 	ast_cli(fd, "--------------------------------------------------------------------------------\n");
 	if ((argc == 2) || (!strcasecmp(argv[1],"audio"))) {
 		found = 1;
-		for (i=0;i<13;i++) {
+		for (i=0;i<AST_FORMAT_MAX_AUDIO_NUM;i++) {
 			snprintf(hex,25,"(0x%x)",1<<i);
 			ast_cli(fd, "%11u (1 << %2d) %10s  audio   %8s   (%s)\n",1 << i,i,hex,ast_getformatname(1<<i),ast_codec2str(1<<i));
 		}
@@ -647,7 +647,7 @@
 
 	if ((argc == 2) || (!strcasecmp(argv[1],"video"))) {
 		found = 1;
-		for (i=18;i<22;i++) {
+		for (i=18;i<AST_FORMAT_MAX_VIDEO_NUM;i++) {
 			snprintf(hex,25,"(0x%x)",1<<i);
 			ast_cli(fd, "%11u (1 << %2d) %10s  video   %8s   (%s)\n",1 << i,i,hex,ast_getformatname(1<<i),ast_codec2str(1<<i));
 		}
@@ -675,7 +675,7 @@
 	ast_cli(fd, "--------------------------------------------------------------------------------\n");
 	if ((argc == 3) || (!strcasecmp(argv[3],"audio"))) {
 		found = 1;
-		for (i=0;i<13;i++) {
+		for (i=0;i<AST_FORMAT_MAX_AUDIO_NUM;i++) {
 			snprintf(hex,25,"(0x%x)",1<<i);
 			ast_cli(fd, "%11u (1 << %2d) %10s  audio   %8s   (%s)\n",1 << i,i,hex,ast_getformatname(1<<i),ast_codec2str(1<<i));
 		}
@@ -691,7 +691,7 @@
 
 	if ((argc == 3) || (!strcasecmp(argv[3],"video"))) {
 		found = 1;
-		for (i=18;i<22;i++) {
+		for (i=18;i<AST_FORMAT_MAX_VIDEO_NUM;i++) {
 			snprintf(hex,25,"(0x%x)",1<<i);
 			ast_cli(fd, "%11u (1 << %2d) %10s  video   %8s   (%s)\n",1 << i,i,hex,ast_getformatname(1<<i),ast_codec2str(1<<i));
 		}
@@ -1405,7 +1405,7 @@
 static int speex_samples(unsigned char *data, int len)
 {
 	static int SpeexSubModeSz[] = {
-               5, 43, 119, 160,
+               0, 43, 119, 160,
 		220, 300, 364, 492, 
 		79, 0, 0, 0,
 		0, 0, 0, 0 };
@@ -1429,7 +1429,8 @@
 		bit += off;
 
 		if ((len * 8 - bit) < 5) {
-			ast_log(LOG_WARNING, "Not enough bits remaining after wide band for speex samples.\n");
+		/* removed warning because it triggers all the time with OpenWengo? */
+		/* ast_log(LOG_WARNING, "Not enough bits remaining after wide band for speex samples.\n"); */
 			break;
 		}
 
@@ -1452,6 +1453,7 @@
 			bit += c * 8;
 		} else if (c > 8) {
 			/* unknown */
+			ast_log(LOG_WARNING, "unknown control type reading speex frame\n");
 			break;
 		} else {
 			/* skip number bits for submode (less the 5 control bits) */
@@ -1462,11 +1464,22 @@
 	return cnt;
 }
 
+int ast_codec_get_rate(struct ast_frame *f)
+{
+        switch(f->subclass) {
+        case AST_FORMAT_SPEEXWB_WENGO:
+                return 16000;
+	}
+
+	return 8000;
+}
+
 int ast_codec_get_samples(struct ast_frame *f)
 {
 	int samples=0;
 	switch(f->subclass) {
 	case AST_FORMAT_SPEEX:
+	case AST_FORMAT_SPEEXWB_WENGO:
 		samples = speex_samples(f->data, f->datalen);
 		break;
 	case AST_FORMAT_G723_1:
diff -r -u asterisk-1.4.10.1/main/rtp.c asterisk-1.4.10.1-wb/main/rtp.c
--- asterisk-1.4.10.1/main/rtp.c	2007-08-06 09:27:24.000000000 -0600
+++ asterisk-1.4.10.1-wb/main/rtp.c	2007-08-22 20:13:24.000000000 -0600
@@ -1010,8 +1010,8 @@
 		rtp->drxcore = (double) rtp->rxcore.tv_sec + (double) rtp->rxcore.tv_usec / 1000000;
 		/* map timestamp to a real time */
 		rtp->seedrxts = timestamp; /* Their RTP timestamp started with this */
-		rtp->rxcore.tv_sec -= timestamp / 8000;
-		rtp->rxcore.tv_usec -= (timestamp % 8000) * 125;
+		rtp->rxcore.tv_sec -= timestamp / ast_codec_get_rate(&rtp->f);
+		rtp->rxcore.tv_usec -= (timestamp % ast_codec_get_rate(&rtp->f)) * ( 1000000 / ast_codec_get_rate(&rtp->f) );
 		/* Round to 0.1ms for nice, pretty timestamps */
 		rtp->rxcore.tv_usec -= rtp->rxcore.tv_usec % 100;
 		if (rtp->rxcore.tv_usec < 0) {
@@ -1023,13 +1023,13 @@
 
 	gettimeofday(&now,NULL);
 	/* rxcore is the mapping between the RTP timestamp and _our_ real time from gettimeofday() */
-	tv->tv_sec = rtp->rxcore.tv_sec + timestamp / 8000;
-	tv->tv_usec = rtp->rxcore.tv_usec + (timestamp % 8000) * 125;
+	tv->tv_sec = rtp->rxcore.tv_sec + timestamp / ast_codec_get_rate(&rtp->f);
+	tv->tv_usec = rtp->rxcore.tv_usec + (timestamp % ast_codec_get_rate(&rtp->f)) * ( 1000000 / ast_codec_get_rate(&rtp->f) );
 	if (tv->tv_usec >= 1000000) {
 		tv->tv_usec -= 1000000;
 		tv->tv_sec += 1;
 	}
-	prog = (double)((timestamp-rtp->seedrxts)/8000.);
+	prog = (double)((timestamp-rtp->seedrxts)/(double)ast_codec_get_rate(&rtp->f));
 	dtv = (double)rtp->drxcore + (double)(prog);
 	current_time = (double)now.tv_sec + (double)now.tv_usec/1000000;
 	transit = current_time - dtv;
@@ -1303,8 +1303,8 @@
 		calc_rxstamp(&rtp->f.delivery, rtp, timestamp, mark);
 		/* Add timing data to let ast_generic_bridge() put the frame into a jitterbuf */
 		rtp->f.has_timing_info = 1;
-		rtp->f.ts = timestamp / 8;
-		rtp->f.len = rtp->f.samples / 8;
+		rtp->f.ts = timestamp / ( ast_codec_get_rate(&rtp->f) / 1000 );
+		rtp->f.len = rtp->f.samples / ( ast_codec_get_rate(&rtp->f) / 1000 );
 	} else {
 		/* Video -- samples is # of samples vs. 90000 */
 		if (!rtp->lastividtimestamp)
@@ -1341,6 +1341,7 @@
 	{{1, AST_FORMAT_ILBC}, "audio", "iLBC"},
 	{{1, AST_FORMAT_G722}, "audio", "G722"},
 	{{1, AST_FORMAT_G726_AAL2}, "audio", "AAL2-G726-32"},
+	{{1, AST_FORMAT_SPEEXWB_WENGO}, "audio", "G726-64wb"},
 	{{0, AST_RTP_DTMF}, "audio", "telephone-event"},
 	{{0, AST_RTP_CISCO_DTMF}, "audio", "cisco-telephone-event"},
 	{{0, AST_RTP_CN}, "audio", "CN"},
@@ -1381,6 +1382,7 @@
 	[97] = {1, AST_FORMAT_ILBC},
 	[99] = {1, AST_FORMAT_H264},
 	[101] = {0, AST_RTP_DTMF},
+	[102] = {1, AST_FORMAT_SPEEXWB_WENGO},
 	[110] = {1, AST_FORMAT_SPEEX},
 	[111] = {1, AST_FORMAT_G726},
 	[112] = {1, AST_FORMAT_G726_AAL2},
@@ -2580,13 +2582,16 @@
 	int pred;
 	int mark = 0;
 
+	if ( !strcmp( f->src, "lintoulaw" ) ) {
+		pred = 0;
+	}
 	ms = calc_txstamp(rtp, &f->delivery);
 	/* Default prediction */
 	if (f->subclass < AST_FORMAT_MAX_AUDIO) {
 		pred = rtp->lastts + f->samples;
 
 		/* Re-calculate last TS */
-		rtp->lastts = rtp->lastts + ms * 8;
+		rtp->lastts = rtp->lastts + ms * ( ast_codec_get_rate(f) / 1000 );
 		if (ast_tvzero(f->delivery)) {
 			/* If this isn't an absolute delivery time, Check if it is close to our prediction, 
 			   and if so, go with our prediction */
@@ -2622,7 +2627,7 @@
 		rtp->lastdigitts = rtp->lastts;
 
 	if (f->has_timing_info)
-		rtp->lastts = f->ts * 8;
+		rtp->lastts = f->ts * ( ast_codec_get_rate(&rtp->f) / 1000 );
 
 	/* Get a pointer to the header */
 	rtpheader = (unsigned char *)(f->data - hdrlen);
diff -r -u asterisk-1.4.10.1/main/translate.c asterisk-1.4.10.1-wb/main/translate.c
--- asterisk-1.4.10.1/main/translate.c	2007-06-06 07:18:39.000000000 -0600
+++ asterisk-1.4.10.1-wb/main/translate.c	2007-08-21 14:41:56.000000000 -0600
@@ -492,7 +492,7 @@
 /*! \brief CLI "show translation" command handler */
 static int show_translation_deprecated(int fd, int argc, char *argv[])
 {
-#define SHOW_TRANS 13
+#define SHOW_TRANS AST_FORMAT_MAX_AUDIO_NUM
 	int x, y, z;
 	int curlen = 0, longest = 0;
 
